@@ -26,6 +26,8 @@ def_selector_args_handler(vector strvec)
 
 	buff = set_value(strvec);
 	conf->default_selector_args = atoi(buff);
+
+	free(buff);
 }
 
 static void
@@ -141,6 +143,20 @@ hw_selector_args_handler(vector strvec)
 
 	buff = set_value(strvec);
 	hwe->selector_args = atoi(buff);
+
+	free(buff);
+}
+
+static void
+hw_path_checker_handler(vector strvec)
+{
+	char * buff;
+	struct hwentry * hwe = VECTOR_SLOT(conf->hwtable, VECTOR_SIZE(conf->hwtable) - 1);
+
+	buff = set_value(strvec);
+	hwe->checker_index = get_checker_id(buff);
+
+	free(buff);
 }
 
 /*
@@ -205,6 +221,8 @@ mp_selector_args_handler(vector strvec)
 
 	buff = set_value(strvec);
 	mpe->selector_args = atoi(buff);
+
+	free(buff);
 }
 			
 vector
@@ -232,6 +250,7 @@ init_keywords(void)
 	install_keyword("getuid_callout", &hw_getuid_callout_handler);
 	install_keyword("path_selector", &hw_selector_handler);
 	install_keyword("path_selector_args", &hw_selector_args_handler);
+	install_keyword("path_checker", &hw_path_checker_handler);
 	install_sublevel_end();
 
 	install_keyword_root("multipaths", &multipaths_handler);
