@@ -25,10 +25,11 @@
 #include <string.h>
 #include <signal.h>
 #include <sys/ioctl.h>
-#include <dlist.h>
-#include <sysfs/libsysfs.h>
 
+#include "../libsysfs/dlist.h"
+#include "../libsysfs/sysfs/libsysfs.h"
 #include "../libdevmapper/libdevmapper.h"
+#include "../libcheckers/checkers.h"
 #include "main.h"
 #include "devinfo.h"
 #include "config.h"
@@ -44,13 +45,6 @@
 #define argis(x) if (0 == strcmp (x, argv[i]))
 #define MATCH(x,y) 0 == strncmp (x, y, strlen(y))
 
-static int
-exit_tool (int status)
-{
-	unlink(RUN);
-	exit(status);
-}
-	
 static int
 devt2devname (char *devname, int major, int minor)
 {
@@ -64,16 +58,16 @@ devt2devname (char *devname, int major, int minor)
 
 	if (sysfs_get_mnt_path(sysfs_path, FILE_NAME_SIZE)) {
 		fprintf(stderr, "-D feature available with sysfs only\n");
-		exit_tool(1);
+		exit(1);
 	}
 		
 	if(safe_sprintf(attr_ref_value, "%i:%i\n", major, minor)) {
 		fprintf(stderr, "attr_ref_value too small\n");
-		exit_tool(1);
+		exit(1);
 	}
 	if(safe_sprintf(block_path, "%s/block", sysfs_path)) {
 		fprintf(stderr, "block_path too small\n");
-		exit_tool(1);
+		exit(1);
 	}
 	sdir = sysfs_open_directory(block_path);
 	sysfs_read_directory(sdir);
@@ -82,7 +76,7 @@ devt2devname (char *devname, int major, int minor)
 		if(safe_sprintf(attr_path, "%s/%s/dev",
 				block_path, devp->name)) {
 			fprintf(stderr, "attr_path too small\n");
-			exit_tool(1);
+			exit(1);
 		}
 		sysfs_read_attribute_value(attr_path, attr_value,
 					   sizeof(attr_value));
@@ -91,7 +85,7 @@ devt2devname (char *devname, int major, int minor)
 			if(safe_sprintf(attr_path, "%s/%s",
 					block_path, devp->name)) {
 				fprintf(stderr, "attr_path too small\n");
-				exit_tool(1);
+				exit(1);
 			}
 			sysfs_get_name_from_path(attr_path, devname,
 						 FILE_NAME_SIZE);
@@ -125,6 +119,35 @@ blacklist (char * dev) {
 	return 0;
 }
 
+static int
+select_checkfn(struct path *pp)
+{
+	char checker_name[CHECKER_NAME_SIZE];
+	int i;
+	struct hwentry * hwe;
+
+	/*
+	 * default checkfn
+	 */
+	pp->checkfn = &readsector0;
+
+	for (i = 0; i < VECTOR_SIZE(conf->hwtable); i++) {
+		hwe = VECTOR_SLOT(conf->hwtable, i);
+		if (MATCH(hwe->vendor, pp->vendor_id) &&
+		    MATCH(hwe->product, pp->product_id) &&
+		    hwe->checker_index > 0) {
+			get_checker_name(checker_name, hwe->checker_index);
+			dbg("set %s path checker for %s",
+				checker_name, pp->dev);
+			pp->checkfn = get_checker_addr(hwe->checker_index);
+			return 0;
+		}
+	}
+	get_checker_name(checker_name, READSECTOR0);
+	dbg("set %s path checker for %s", checker_name, pp->dev);
+	return 0;
+}
+
 static int
 devinfo (struct path *curpath)
 {
@@ -143,8 +166,15 @@ devinfo (struct path *curpath)
 	 * then those not available through sysfs
 	 */
 	get_serial(curpath->serial, curpath->dev_t);
-	curpath->tur = do_tur(curpath->dev_t);
 	curpath->claimed = get_claimed(curpath->dev_t);
+
+	/*
+	 * get path state
+	 */
+	select_checkfn(curpath);
+	curpath->state = checkpath(curpath->sg_dev_t, curpath->checkfn);
+dbg("state : %i", curpath->state);
+dbg("sg_dev_t : %s", curpath->sg_dev_t);
 	
 	/*
 	 * get path prio
@@ -152,7 +182,7 @@ devinfo (struct path *curpath)
 	if(safe_sprintf(buff, "%s /block/%s",
 			conf->default_getprio, curpath->dev)) {
 		fprintf(stderr, "buff too small\n");
-		exit_tool(1);
+		exit(1);
 	}
 
 	dbg("get prio callout :");
@@ -182,7 +212,7 @@ devinfo (struct path *curpath)
 			if(safe_sprintf(buff, "%s /block/%s",
 				 hwe->getuid, curpath->dev)) {
 				fprintf(stderr, "buff too small\n");
-				exit_tool(1);
+				exit(1);
 			}
 			if (execute_program(buff, curpath->wwid,
 			    WWID_SIZE) == 0) {
@@ -219,7 +249,7 @@ devinfo (struct path *curpath)
 	if(safe_sprintf(buff, "%s /block/%s",
 			conf->default_getuid, curpath->dev)) {
 		fprintf(stderr, "buff too small\n");
-		exit_tool(1);
+		exit(1);
 	}
 
 	dbg("default get uid callout :");
@@ -252,12 +282,12 @@ get_pathvec_sysfs (vector pathvec)
 
 	if (sysfs_get_mnt_path(sysfs_path, FILE_NAME_SIZE)) {
 		fprintf(stderr, "multipath tools need sysfs\n");
-		exit_tool(1);
+		exit(1);
 	}
 	
 	/*
-	 * if called from /etc/dev.d , only consider the paths that relate
-	 * to the device pointed by conf->dev
+	 * if called from /etc/dev.d or pathcheckers, only consider the paths
+	 * that relate to the device pointed by conf->dev
 	 */
 	if (conf->dev != NULL && filepresent(conf->dev)) {
 		curpath = zalloc(sizeof (struct path));
@@ -281,7 +311,7 @@ get_pathvec_sysfs (vector pathvec)
 		curpath = zalloc(sizeof (struct path));
 		if(safe_sprintf(curpath->dev, "%s", buff)) {
 			fprintf(stderr, "curpath->dev too small\n");
-			exit_tool(1);
+			exit(1);
 		}
 
 		if (devinfo(curpath))
@@ -293,7 +323,7 @@ get_pathvec_sysfs (vector pathvec)
 		
 	if(safe_sprintf(path, "%s/block", sysfs_path)) {
 		fprintf(stderr, "path too small\n");
-		exit_tool(1);
+		exit(1);
 	}
 	sdir = sysfs_open_directory(path);
 	sysfs_read_directory(sdir);
@@ -319,7 +349,7 @@ get_pathvec_sysfs (vector pathvec)
 
 		if(safe_sprintf(curpath->dev, "%s", buff)) {
 			fprintf(stderr, "curpath->dev too small\n");
-			exit_tool(1);
+			exit(1);
 		}
 		if (devinfo(curpath)) {
 			free (curpath);
@@ -423,7 +453,7 @@ print_path (struct path * pp, int style)
 
 	printf ("%s ", pp->dev);
 
-	if (pp->tur)
+	if (pp->state)
 		printf ("[ready ] ");
 	else
 		printf ("[faulty] ");
@@ -550,14 +580,14 @@ coalesce_paths (vector mp, vector pathvec)
 		vector_set_slot (mpp->paths, VECTOR_SLOT(pathvec, k));
 
 		if (mpp->size == 0)
-			mpp->size = get_disk_size (pp1->dev);
+			mpp->size = get_disk_size(pp1->dev);
 
 		for (i = k + 1; i < VECTOR_SIZE(pathvec); i++) {
 			pp2 = VECTOR_SLOT(pathvec, i);
-			if (0 == strcmp (pp1->wwid, pp2->wwid)) {
-				vector_alloc_slot (mpp->paths);
-				vector_set_slot (mpp->paths,
-						 VECTOR_SLOT(pathvec, i));
+			if (0 == strcmp(pp1->wwid, pp2->wwid)) {
+				vector_alloc_slot(mpp->paths);
+				vector_set_slot(mpp->paths,
+						VECTOR_SLOT(pathvec, i));
 			}
 		}
 		vector_set_slot(mp, mpp);
@@ -569,16 +599,16 @@ dm_simplecmd (int task, const char *name) {
 	int r = 0;
 	struct dm_task *dmt;
 
-	if (!(dmt = dm_task_create (task)))
+	if (!(dmt = dm_task_create(task)))
 		return 0;
 
-	if (!dm_task_set_name (dmt, name))
+	if (!dm_task_set_name(dmt, name))
 		goto out;
 
-	r = dm_task_run (dmt);
+	r = dm_task_run(dmt);
 
 	out:
-		dm_task_destroy (dmt);
+		dm_task_destroy(dmt);
 		return r;
 }
 
@@ -586,20 +616,20 @@ static int
 dm_addmap (int task, const char *name, const char *params, unsigned long size) {
 	struct dm_task *dmt;
 
-	if (!(dmt = dm_task_create (task)))
+	if (!(dmt = dm_task_create(task)))
 		return 0;
 
-	if (!dm_task_set_name (dmt, name))
-		goto addout;
+	if (!dm_task_set_name(dmt, name))
+		goto out;
 
-	if (!dm_task_add_target (dmt, 0, size, DM_TARGET, params))
-		goto addout;
+	if (!dm_task_add_target(dmt, 0, size, DM_TARGET, params))
+		goto out;
 
-	if (dm_task_run (dmt))
+	if (dm_task_run(dmt))
 		return 0;
 
-	addout:
-	dm_task_destroy (dmt);
+	out:
+	dm_task_destroy(dmt);
 	return 1;
 }
 
@@ -611,13 +641,13 @@ dm_map_present (char * str)
         struct dm_names *names;
         unsigned next = 0;
 
-	if (!(dmt = dm_task_create (DM_DEVICE_LIST)))
+	if (!(dmt = dm_task_create(DM_DEVICE_LIST)))
 		return 0;
 
-	if (!dm_task_run (dmt))
+	if (!dm_task_run(dmt))
 		goto out;
 
-	if (!(names = dm_task_get_names (dmt)))
+	if (!(names = dm_task_get_names(dmt)))
 		goto out;
 
 	if (!names->dev) {
@@ -625,14 +655,14 @@ dm_map_present (char * str)
 	}
 
 	do {
-		if (0 == strcmp (names->name, str))
+		if (0 == strcmp(names->name, str))
 			r = 1;
 		next = names->next;
 		names = (void *) names + next;
 	} while (next);
 
 	out:
-	dm_task_destroy (dmt);
+	dm_task_destroy(dmt);
 	return r;
 }
 
@@ -662,7 +692,7 @@ dm_get_map(char * name, char * outparams)
 	next = dm_get_next_target(dmt, next, &start, &length,
 				  &target_type, &params);
 
-	if (snprintf(outparams, PARAMS_SIZE, params) >= PARAMS_SIZE)
+	if (safe_snprintf(outparams, PARAMS_SIZE, params))
 		goto out;
 
 	r = 1;
@@ -675,7 +705,8 @@ dm_get_map(char * name, char * outparams)
 static int
 dm_reinstate(char * mapname, char * path)
 {
-        int r = 0, sz = 1, i;
+        int r = 0;
+	int sz;
         struct dm_task *dmt;
         char *str;
 
@@ -688,7 +719,7 @@ dm_reinstate(char * mapname, char * path)
         if (!dm_task_set_sector(dmt, 0))
                 goto out;
 
-	sz = strlen(path) + 10;
+	sz = strlen(path) + 11;
         str = zalloc(sz);
 
 	snprintf(str, sz, "reinstate %s\n", path);
@@ -770,7 +801,7 @@ assemble_map (struct multipath * mp)
 
 	if (shift >= freechar) {
 		fprintf(stderr, "mp->params too small\n");
-		exit_tool(1);
+		exit(1);
 	}
 	p += shift;
 	freechar -= shift;
@@ -781,7 +812,7 @@ assemble_map (struct multipath * mp)
 			         selector, VECTOR_SIZE(pgpaths), selector_args);
 		if (shift >= freechar) {
 			fprintf(stderr, "mp->params too small\n");
-			exit_tool(1);
+			exit(1);
 		}
 		p += shift;
 		freechar -= shift;
@@ -791,7 +822,7 @@ assemble_map (struct multipath * mp)
 					 (char *)VECTOR_SLOT(pgpaths, j));
 			if (shift >= freechar) {
 				fprintf(stderr, "mp->params too small\n");
-				exit_tool(1);
+				exit(1);
 			}
 			p += shift;
 			freechar -= shift;
@@ -799,7 +830,7 @@ assemble_map (struct multipath * mp)
 	}
 	if (freechar < 1) {
 		fprintf(stderr, "mp->params too small\n");
-		exit_tool(1);
+		exit(1);
 	}
 	snprintf(p, 1, "\n");
 }
@@ -819,7 +850,6 @@ setup_map (vector pathvec, vector mp, int slot)
 	int op;
 
 	mpp = VECTOR_SLOT(mp, slot);
-	pp = VECTOR_SLOT(mpp->paths, 0);
 
 	/*
 	 * don't bother if devmap size is unknown
@@ -837,6 +867,7 @@ setup_map (vector pathvec, vector mp, int slot)
 		if (pp->claimed)
 			return 0;
 	}
+	pp = VECTOR_SLOT(mpp->paths, 0);
 
 	/*
 	 * iopolicy selection logic :
@@ -849,43 +880,55 @@ setup_map (vector pathvec, vector mp, int slot)
 	dbg("iopolicy selector :");
 	dbg("===================");
 
-	/* 1) set internal default */
+	/*
+	 * 1) set internal default
+	 */
 	iopolicy = FAILOVER;
 	get_pgpolicy_name(iopolicy_name, FAILOVER);
 	dbg("internal default)\tiopolicy = %s", iopolicy_name);
 
-	/* 2) override by config file default */
-	if (conf->default_iopolicy >= 0) {
+	/*
+	 * 2) override by config file default
+	 */
+	if (conf->default_iopolicy > 0) {
 		iopolicy = conf->default_iopolicy;
 		get_pgpolicy_name(iopolicy_name, iopolicy);
 		dbg("config file default)\tiopolicy = %s", iopolicy_name);
 	}
 	
-	/* 3) override by controler wide setting */
+	/*
+	 * 3) override by controler wide setting
+	 */
 	for (i = 0; i < VECTOR_SIZE(conf->hwtable); i++) {
 		hwe = VECTOR_SLOT(conf->hwtable, i);
 
 		if (MATCH (pp->vendor_id, hwe->vendor) &&
-		    MATCH (pp->product_id, hwe->product)) {
+		    MATCH (pp->product_id, hwe->product) &&
+		    hwe->iopolicy > 0) {
 			iopolicy = hwe->iopolicy;
 			get_pgpolicy_name(iopolicy_name, iopolicy);
 			dbg("controler override)\tiopolicy = %s", iopolicy_name);
 		}
 	}
 
-	/* 4) override by LUN specific setting */
+	/*
+	 * 4) override by LUN specific setting
+	 */
 	for (i = 0; i < VECTOR_SIZE(conf->mptable); i++) {
 		mpe = VECTOR_SLOT(conf->mptable, i);
 
-		if (strcmp(mpe->wwid, mpp->wwid) == 0) {
+		if (strcmp(mpe->wwid, mpp->wwid) == 0 &&
+		    mpe->iopolicy > 0) {
 			iopolicy = mpe->iopolicy;
 			get_pgpolicy_name(iopolicy_name, iopolicy);
 			dbg("lun override)\t\tiopolicy = %s", iopolicy_name);
 		}
 	}
 
-	/* 5) cmd line flag has the last word */
-	if (conf->iopolicy_flag >= 0) {
+	/*
+	 * 5) cmd line flag has the last word
+	 */
+	if (conf->iopolicy_flag > 0) {
 		iopolicy = conf->iopolicy_flag;
 		get_pgpolicy_name(iopolicy_name, iopolicy);
 		dbg("cmd flag override)\tiopolicy = %s", iopolicy_name);
@@ -897,10 +940,14 @@ setup_map (vector pathvec, vector mp, int slot)
 	 *  2) apply selected grouping policy to paths
 	 */
 
-	/* 1) sort by priority */
+	/*
+	 * 1) sort by priority
+	 */
 	mpp->paths = sort_pathvec_by_prio(mpp->paths);
 
-	/* 2) apply selected grouping policy */
+	/*
+	 * 2) apply selected grouping policy
+	 */
 	if (iopolicy == MULTIBUS)
 		one_group (mpp);
 
@@ -959,7 +1006,14 @@ setup_map (vector pathvec, vector mp, int slot)
 		if (conf->dev != NULL && filepresent(conf->dev) &&
 		    dm_get_map(mapname, curparams) &&
 		    0 == strncmp(mpp->params, curparams, strlen(mpp->params))) {
-			dm_reinstate(mapname, conf->dev);
+	                pp = zalloc(sizeof(struct path));
+	                basename(conf->dev, pp->dev);
+
+	                if (devinfo(pp))
+	                        return 1;
+
+			dm_reinstate(mapname, pp->dev_t);
+	                free(pp);
 
 			if (conf->verbosity > 1)
 				printf("[reinstate %s in %s]\n",
@@ -998,8 +1052,10 @@ signal_daemon (void)
 
 	file = fopen (PIDFILE, "r");
 
-	if (!file && conf->verbosity > 0) {
-		fprintf (stderr, "cannot signal daemon, pidfile not found\n");
+	if (!file) {
+		if (conf->verbosity > 0)
+			fprintf (stderr, "cannot signal daemon, "
+					 "pidfile not found\n");
 		return;
 	}
 
@@ -1104,7 +1160,40 @@ usage (char * progname)
 		"\t\t\t(udev-style $DEVNAME reference, eg /dev/sdb)\n" \
 		);
 
-	exit_tool(1);
+	exit(1);
+}
+
+int try_lock (char * file)
+{
+	int fd;
+	struct flock fl;
+
+	/*
+	 * create the file to lock if it does not exist
+	 */
+	fd = open(file, O_CREAT|O_RDWR);
+
+	if (fd < 0) {
+		fprintf(stderr, "can't create runfile\n");
+		exit(1);
+	}
+	fl.l_type = F_WRLCK;
+	fl.l_whence = 0;
+	fl.l_start = 0;
+	fl.l_len = 0;
+
+	/*
+	 * set a max wait time
+	 */
+	alarm(2);
+	
+	if (fcntl(fd, F_SETLKW, &fl) == -1) {
+		fprintf(stderr, "can't take a write lease on %s\n", file);
+		return 1;
+	}
+	alarm(0);
+
+	return 0;
 }
 
 int
@@ -1113,30 +1202,19 @@ main (int argc, char *argv[])
 	vector mp;
 	vector pathvec;
 	int k;
-	int try = 0;
 	int arg;
 	extern char *optarg;
 	extern int optind;
 
 	/*
 	 * Don't run in parallel
+	 * ie, acquire a F_WRLCK-type lock on RUN
 	 */
-	while (filepresent(RUN) && try++ < MAXTRY)
-		usleep(100000);
-
-	if (filepresent(RUN)) {
+	if (try_lock(RUN)) {
 		fprintf(stderr, "waited for to long. exiting\n");
 		exit(1);
 	}
 	
-	/*
-	 * Our turn
-	 */
-	if (!open(RUN, O_CREAT)) {
-		fprintf(stderr, "can't create runfile\n");
-		exit(1);
-	}
-		
 	/*
 	 * alloc config struct
 	 */
@@ -1147,14 +1225,14 @@ main (int argc, char *argv[])
 	 */
 	conf->dry_run = 0;		/* 1 == Do not Create/Update devmaps */
 	conf->verbosity = 1;		/* 1 == Print mp names */
-	conf->iopolicy_flag = -1;	/* do not override defaults */
+	conf->iopolicy_flag = 0;	/* do not override defaults */
 	conf->major = -1;
 	conf->minor = -1;
 	conf->signal = 1;		/* 1 == Send a signal to multipathd */
 	conf->dev = zalloc(sizeof(char) * FILE_NAME_SIZE);
 	conf->default_selector = NULL;
 	conf->default_selector_args = 0;
-	conf->default_iopolicy = -1;
+	conf->default_iopolicy = 0;
 	conf->mptable = NULL;
 	conf->hwtable = NULL;
 	conf->blist = NULL;
@@ -1214,7 +1292,7 @@ main (int argc, char *argv[])
 	/*
 	 * read the config file
 	 */
-	if (filepresent (CONFIGFILE))
+	if (filepresent(CONFIGFILE))
 		init_data (CONFIGFILE, init_keywords);
 	
 	/*
@@ -1248,15 +1326,15 @@ main (int argc, char *argv[])
 
 	if (mp == NULL || pathvec == NULL) {
 		fprintf(stderr, "can not allocate memory\n");
-		exit_tool(1);
+		exit(1);
 	}
 
 	if (get_pathvec_sysfs(pathvec))
-		exit_tool(1);
+		exit(1);
 
 	if (VECTOR_SIZE(pathvec) == 0 && conf->verbosity > 0) {
-		fprintf (stdout, "no path found\n");
-		exit_tool(0);
+		fprintf(stdout, "no path found\n");
+		exit(0);
 	}
 
 	coalesce_paths(mp, pathvec);
@@ -1267,10 +1345,10 @@ main (int argc, char *argv[])
 	}
 
 	if (conf->verbosity > 1)
-		fprintf (stdout, "#\n# device maps :\n#\n");
+		fprintf(stdout, "#\n# device maps :\n#\n");
 
 	for (k = 0; k < VECTOR_SIZE(mp); k++)
-		setup_map (pathvec, mp, k);
+		setup_map(pathvec, mp, k);
 
 	/*
 	 * signal multipathd that new devmaps may have come up
@@ -1281,13 +1359,8 @@ main (int argc, char *argv[])
 	/*
 	 * free allocs
 	 */
-	free (mp);
-	free (pathvec);
+	free(mp);
+	free(pathvec);
 
-	/*
-	 * release runfile
-	 */
-	unlink (RUN);
-
-	exit (0);
+	exit(0);
 }
