@@ -37,7 +37,6 @@
 #define DEFAULT_GETUID	"/sbin/scsi_id -g -u -s"
 #define DEFAULT_UDEVDIR	"/udev"
 #define DEFAULT_SELECTOR	"round-robin"
-#define MAXTRY		50
 
 /*
  * SCSI strings sizes
@@ -74,6 +73,7 @@ struct scsi_dev {
 struct path {
 	char dev[FILE_NAME_SIZE];
 	char dev_t[FILE_NAME_SIZE];
+	char sg_dev_t[FILE_NAME_SIZE];
 	struct scsi_idlun scsi_id;
 	struct sg_id sg_id;
 	char wwid[WWID_SIZE];
@@ -81,9 +81,10 @@ struct path {
 	char product_id[SCSI_PRODUCT_SIZE];
 	char rev[SCSI_REV_SIZE];
 	char serial[SERIAL_SIZE];
-	int tur;
+	int state;
 	unsigned int priority;
 	int claimed;
+	int (*checkfn) (char *);
 };
 
 struct multipath {
