@@ -16,13 +16,13 @@
 #include <sys/mman.h>
 #include "libsysfs/sysfs/libsysfs.h"
 #include "libsysfs/dlist.h"
+#include "libcheckers/checkers.h"
 
 #include "hwtable.h"
 #include "dict.h"
 #include "parser.h"
 #include "vector.h"
 #include "devinfo.h"
-#include "checkers.h"
 #include "memory.h"
 #include "copy.h"
 #include "log.h"
@@ -35,12 +35,13 @@
 #define MAPNAMESIZE 64
 #define TARGETTYPESIZE 16
 #define PARAMSSIZE 2048
+#define DEV_T_SIZE 2048
 
 #define PIDFILE "/var/run/multipathd.pid"
 #define CONFIGFILE "/etc/multipath.conf"
 #define CALLOUT_DIR "/var/cache/multipathd"
 
-#define MATCH(x, y) strncmp(x, y, strlen(y)) == 0
+#define MATCH(x, y) strncmp(x, y, strlen(x)) == 0
 
 #ifdef CLONE_NEWNS
 #ifdef ia64
@@ -62,8 +63,7 @@ pthread_cond_t *event;
  */
 struct path
 {
-	int major;
-	int minor;
+	char sg_dev_t[DEV_T_SIZE];
 	int (*checkfn) (char *);
 };
 
@@ -84,17 +84,6 @@ struct event_thread
 /*
  * helpers functions
  */
-static int
-makenode (char *devnode, int major, int minor)
-{
-	dev_t dev;
-	
-	dev = makedev (major, minor);
-        unlink (devnode);
-	
-	return mknod(devnode, S_IFCHR | S_IRUSR | S_IWUSR, dev);
-}
-
 static int
 blacklist (char * dev) {
 	int i;
@@ -140,6 +129,7 @@ select_checkfn(struct path *path_p, char *devname)
 	char vendor[8];
 	char product[16];
 	char rev[4];
+	char checker_name[CHECKER_NAME_SIZE];
 	int i, r;
 	struct hwentry * hwe;
 
@@ -158,14 +148,16 @@ select_checkfn(struct path *path_p, char *devname)
 	for (i = 0; i < VECTOR_SIZE(hwtable); i++) {
 		hwe = VECTOR_SLOT(hwtable, i);
 		if (MATCH(hwe->vendor, vendor) &&
-		    MATCH(hwe->product, product)) {
+		    MATCH(hwe->product, product) &&
+		    hwe->checker_index > 0) {
+			get_checker_name(checker_name, hwe->checker_index);
 			LOG (2, "set %s path checker for %s",
-			     checker_list[hwe->checker_index].name,
-			     devname);
-			path_p->checkfn = checker_list[hwe->checker_index].checker;
+			     checker_name, devname);
+			path_p->checkfn = get_checker_addr(hwe->checker_index);
+			return 0;
 		}
 	}
-
+	LOG (2, "set readsector0 path checker for %s", devname);
 	return 0;
 }
 
@@ -271,34 +263,6 @@ out:
 	return devmaps;
 }
 
-static int
-checkpath (struct path *path_p)
-{
-	char devnode[FILENAMESIZE];
-	int r;
-	
-	if (safe_sprintf(devnode, "/tmp/.checker.%i.%i",
-			 path_p->major, path_p->minor)) {
-		fprintf(stderr, "checkpath: devnode too small\n");
-		return -1;
-	}
-	r = makenode(devnode, path_p->major, path_p->minor);
-
-	if (r < 0) {
-		LOG (2, "can not create %s", devnode);
-		return r;
-	}
-
-	r = path_p->checkfn(devnode);
-	unlink(devnode);
-				
-	LOG (2, "check path %i:%i => %s",
-	     path_p->major, path_p->minor,
-	     r ? "up" : "down");
-
-	return r;
-}
-
 static int
 updatepaths (struct paths *failedpaths)
 {
@@ -347,11 +311,14 @@ updatepaths (struct paths *failedpaths)
 		}
 
 		path_p = MALLOC(sizeof (struct path));
-		sscanf(attr_buff, "%i:%i", &path_p->major, &path_p->minor);
-
-		if (!select_checkfn(path_p, devp->name) && checkpath(path_p)) {
-			LOG(2, "discard %i:%i as valid path",
-			    path_p->major, path_p->minor);
+		if (safe_snprintf(path_p->sg_dev_t, DEV_T_SIZE, "%s",
+				  attr_buff)) {
+			fprintf(stderr, "sg_dev_t too small\n");
+			return 1;
+		}
+		if (!select_checkfn(path_p, devp->name) &&
+		    checkpath(path_p->sg_dev_t, path_p->checkfn)) {
+			LOG(2, "discard %s as valid path", path_p->sg_dev_t);
 			FREE(path_p);
 			continue;
 		}
@@ -359,8 +326,7 @@ updatepaths (struct paths *failedpaths)
 		vector_alloc_slot(failedpaths->pathvec);
 		vector_set_slot(failedpaths->pathvec, path_p);
 
-		LOG(2, "%i:%i added to failedpaths",
-		    path_p->major, path_p->minor);
+		LOG(2, "%s added to failedpaths", path_p->sg_dev_t);
 	}
 	pthread_mutex_unlock(failedpaths->lock);
 	sysfs_close_directory(sdir);
@@ -454,7 +420,7 @@ log_failedpaths_vec (vector pathvec)
 
 	for (i = 0; i < VECTOR_SIZE(pathvec); i++) {
 		pp = VECTOR_SLOT(pathvec, i);
-		LOG(1, "%i:%i", pp->major, pp->minor);
+		LOG(1, "%s", pp->sg_dev_t);
 	}
 }
 
@@ -598,16 +564,18 @@ checkerloop (void *ap)
 		for (i = 0; i < VECTOR_SIZE(failedpaths->pathvec); i++) {
 			path_p = VECTOR_SLOT(failedpaths->pathvec, i);
 			
-			if (checkpath(path_p)) {
+			if (checkpath(path_p->sg_dev_t, path_p->checkfn)) {
 				/*
 				 * tell waiterloop we have an event
 				 */
-				LOG(1, "path checker event on device %i:%i",
-					path_p->major, path_p->minor);
+				LOG(1, "path checker event on device %s",
+					path_p->sg_dev_t);
 				pthread_mutex_lock (event_lock);
 				pthread_cond_signal(event);
 				pthread_mutex_unlock (event_lock);
-			}
+			} else
+				LOG (2, "check path %s => down",
+				     path_p->sg_dev_t);
 		}
 		pthread_mutex_unlock(failedpaths->lock);
 		sleep(checkint ? checkint : CHECKINT);
